#include "swmem.h"
#include <stdlib.h>
#include <string.h>


/*********************
 *  "" *
 *********************/

//  
CSMessage::CSMessage(long int id, long int src)
{
	this->id	= id;
	this->src	= src;
}



/*****************************
 *  " " *
 *****************************/

//    
CSQueue::CSQueueItem::CSQueueItem(CSMessage *msg)
{
	this->msg = msg;
	next = NULL;
}

//   
CSQueue::CSQueue()
{
	first = NULL;
	last  = NULL;
}

//  
CSQueue::~CSQueue()
{
	CSQueueItem *item;
	while(!IsEmpty()) {
		item = first;
		first = item->next;
		delete item;
	}
}

//   
bool CSQueue::IsEmpty()
{
	return (first == NULL);
}

//     
bool CSQueue::AddMessage(CSMessage *msg)
{
	CSQueueItem *item = new CSQueueItem(msg);

	if (IsEmpty()) {				//   ,   
		first = last = item;
	} else {						//     
		last->next = item;
		last = item;
	}
	return item == NULL;			//    ,   
}

//     
CSMessage *CSQueue::GetMessage()
{
	if (IsEmpty()) return NULL;		//     

	CSQueueItem *item = first;
	CSMessage *msg = item->msg;

	if (!first->next) {				//   ,  
		first = last = NULL;
	} else {						//   ,   first
		first = first->next;
	}
	
	delete item;
	return msg;
}


/****************************
 *  " " *
 ****************************/


CSStateArray::CSStateArray(int size, int num_arr[], char *cap_arr[])
{
	this->size = size;
	data = new CSAutoState [size];
	
	for (int i=0; i<size; i++) {
		data[i].number	= num_arr[i];
		data[i].caption	= cap_arr[i];
	}
}


CSStateArray::~CSStateArray()
{
	delete [] data;
}


char *CSStateArray::Caption(int state)
{
	return data[BinSearch(state)].caption;
}


//     
int CSStateArray::BinSearch(int key)
{
	int low = 0;
	int high = size - 1;
	int mid;

	while (low <= high) {
		mid = (low + high) / 2;
		if (key < data[mid].number) high = mid - 1;
		else if (key > data[mid].number) low = mid+1;
		else return mid;

	}
	 
	return -1;
}



/*******************
 *  "" *
 *******************/

CSAutomate::CSAutomate(CSQueue *queue, char *automate_id,
					   CSStateArray *st_arr, FILE *f_rec, bool recording)
{
	this->AutomateID	= new char(10);
	strcpy(AutomateID, automate_id);
	this->Queue			= queue;
	this->st_arr		= st_arr;
	this->f_rec			= f_rec;
	this->recording		= recording;
	y_n = 0; Update();
}


CSAutomate::~CSAutomate()
{
 	delete st_arr;
	delete AutomateID;
}


//   
void CSAutomate::StartRec()
{
	recording = true;
}


//    
void CSAutomate::PrintRec()
{
	if (!recording) return;		// " " ,   .

	char *s_old = st_arr->Caption(y_o);
	char *s_new = st_arr->Caption(y_n);

	fprintf(f_rec, "\n{ %s: %2.1d '%s' -> %2.1d '%s',   %2.1d }",
		AutomateID, y_o, s_old, y_n, s_new, action);
}


//   
void CSAutomate::StopRec()
{
	recording = false;
}


//    
void CSAutomate::Update()
{
	y_o = y_n;
	priority = -1;
	action = -1;
}

//   
int CSAutomate::GetState()
{
	return y_o;
}

